<?php

class ThumbSniper
{

	protected $options;

	const GUNSLINGER_URL = "http://gunslinger.thumbsniper.com/fire.php";
	const JQUERY_PATH = "jquery/jquery-1.8.0.min.js";
	const QTIP_PATH = "qtip/jquery.qtip.min.js";
	const QTIP_CSS_PATH = "qtip/jquery.qtip.min.css";



	function ThumbSniper()
	{
		$this->__construct();
	} // function



	function __construct()
	{
		$this->options = new ThumbSniper_Options;
		$this->init();
	} // function



	function admin_init()
	{

	} // function



	function init()
	{
		add_action(
			$tag = 'admin_init',
			$function_to_add = array( &$this, 'admin_init' )
			);

		add_action(
			$tag = 'plugins_loaded',
			$function_to_add = array( &$this, 'add_plugin' )
			);

		if( $this->options->get_general_setting( 'thumbsniper_showfooter' ) == 'yes' )
		{
			add_action(
				$tag = 'wp_footer',
				$function_to_add = array( &$this, 'thumbsniper_footer' )
				);
		}
	} // function



	function add_plugin()
	{
		if( !is_admin() )
		{
			global $post;

			if( !isset($post) || get_post_meta($post->ID, 'thumbsniper_hide', true ) == "" )
			{
				if( $this->options->get_general_setting('thumbsniper_wp_jquery') == "no" )
				{
					wp_deregister_script(
						$handle = 'jquery'
						);

					wp_register_script(
						$handle = 'jquery',
						$src = plugins_url( self::JQUERY_PATH, __FILE__ ),
						$deps = false,
						$ver = false,
						$in_footer = false
						);

					wp_enqueue_script(
						$handle = 'jquery'
						);
				}

				wp_enqueue_script(
					$handle = 'qtip', 
					$src = plugins_url( self::QTIP_PATH, __FILE__ ),
					$deps = array('jquery'),
					$ver = false,
					$in_footer = $this->options->get_general_setting('thumbsniper_qtip_footer') == "yes" ? true : false
				);

				add_action(
					$tag = 'wp_head',
					$function_to_add = array( &$this, 'thumbsniper_styles' )
					);

				add_action(
					$tag = 'wp_footer',
					$function_to_add = array( &$this, 'thumbsniper_scripts' )
					);
			}
		}
	} // function



	function thumbsniper_styles()
	{
		$out = "\n<!-- ThumbSniper styles - start -->\n";

		global $post;

		if( !isset($post) || get_post_meta($post->ID, 'thumbsniper_hide', true ) == "" )
		{
			$out.= '<link rel="stylesheet" href="' . plugins_url( self::QTIP_CSS_PATH, __FILE__ ) . '" />' . "\n";

			//TODO: premium entfernen
			$out.= '<style type="text/css">

					/* ThumbSniper styles */

					img.thumbsniper {
						padding: 0px;
						background: none;
						margin: 0px;
						border: none;
					}

					.ui-tooltip, .qtip {
						position: absolute;
						left: -28000px;
						top: -28000px;
						display: none;
						max-width: 550px;
						min-width: 50px;
						font-size: 10.5px;
						z-index: 15000;
						line-height: inherit;
					}

					.ui-tooltip-content {
						position: relative;
						padding: 15px 15px;
						overflow: hidden;
						border-width: 1px;
						border-style: solid;
						text-align: center;
						word-wrap: break-word;
						overflow: hidden;
					}

					.thumbsniperimageholder {
						position:relative;
						display:table;
						vertical-align: middle;
						text-align: center;
					}

					.thumbsniperinner {
						width:100%;
						display:table-cell;
						vertical-align:middle;
						position:relative;
						text-align:center;
						border: 0px;
					}

					.thumbsniperinnerimg{
						display:inline;
						vertical-align:middle;
					}

					.thumbsniperpremium {
						float: left;
					}
				</style>';
		}

		$out.= "\n<!-- ThumbSniper styles - end -->\n";
		print( $out );
	}



	function thumbsniper_scripts()
	{
		$out = "\n<!-- ThumbSniper scripts - start -->\n";

		global $post;

		$thumbsniper_include_pages = $this->options->get_general_setting( 'thumbsniper_include_pages' );
		$page_included = true;

		if(is_array($thumbsniper_include_pages) && !empty($thumbsniper_include_pages))
		{
			if($post && !in_array($post->ID, $thumbsniper_include_pages))
			{
				$page_included = false;
			}
		}

		if($page_included && (!isset($post) || get_post_meta($post->ID, 'thumbsniper_hide', true ) == "" ))
		{
			$size = $this->options->get_image_size();
			$imageWidth = $size['w'];
			$imageHeight = $size['h'];

			switch( $this->options->get_general_setting( "thumbsniper_position" ) ) 
			{
				case "above":
					$positionMy = "bottom center";
					$positionAt = "top center";
					break;
				case "below":
					$positionMy = "top center";
					$positionAt = "bottom center";
					break;
				case "left":
					$positionMy = "right center";
					$positionAt = "left center";
					break;
				case "right":
					$positionMy = "left center";
					$positionAt = "right center";
					break;
				default:
					$positionMy = "top center";
					$positionAt = "bottom center";
			}

			$out.= '<script type="text/javascript">

				function thumbsniper_rel_to_abs(url){';

					/* Only accept commonly trusted protocols:
					* Only data-image URLs are accepted, Exotic flavours (escaped slash,
					* html-entitied characters) are not supported to keep the function fast.
					* Found here: http://stackoverflow.com/a/7544757
					*/

				$out.= 'if(/^.*\#$/.test(url)) {
						return ""; //only anchor = return nothing
					}

					var qMark = String(url).search(/\?/);
					if(qMark != -1) {
						url = String(url).substring(0, qMark);
					}

					if(/^(https?|file|ftps?|mailto|javascript|data:image\/[^;]{2,9};):/i.test(url)) {
						return url; //Url is already absolute
					}

					var base_url = location.href.match(/^(.+)\/?(?:#.+)?$/)[0]+"/";
					if(url.substring(0,2) == "//") {
						return location.protocol + url;
					}else if(url.charAt(0) == "/") {
						return location.protocol + "//" + location.host + url;
					}else if(url.substring(0,2) == "./") {
						url = "." + url;
					}else if(/^\s*$/.test(url)) {
						return ""; //Empty = Return nothing
					}else {
						url = "../" + url;
					}

					url = base_url + url;
					//var i=0;
					while(/\/\.\.\//.test(url = url.replace(/[^\/]+\/+\.\.\//g,"")));

					/* Escape certain characters to prevent XSS */
					url = url.replace(/\.$/,"").replace(/\/\./g,"").replace(/"/g,"%22").replace(/\'/g,"%27").replace(/</g,"%3C").replace(/>/g,"%3E");

					return url;
				}' . "\n";

				if( $this->options->get_general_setting( 'thumbsniper_jquery_noconflict' ) == "yes" )
				{
					$out.= 'var ThumbSniper = jQuery.noConflict();';
					$jQueryName = "ThumbSniper";
				}else
				{
					$jQueryName = "jQuery";
				}

				$out.= $jQueryName . '(document).ready(function() {' . $jQueryName . '(function() {';

				$thumbsniper_preview = $this->options->get_general_setting( 'thumbsniper_preview' );

				$out.= 'var imageWidth = ' . $imageWidth . ';';
				$out.= 'var imageHeight = ' . $imageHeight . ';';

				if($thumbsniper_preview == 'all' || $thumbsniper_preview == 'external')
				{
					$out.= 'for (var i = 0; i < document.links.length; ++i) {
						var link = document.links[i];
						var blogurl = "' . get_option("siteurl") . '";
						var linkurl = String(link.href).substring(0, blogurl.length);
						var linkproto = String(link.href).substring(0, 4);';

					$out.= 'if(linkproto == "http" || linkproto == "https") {';

					$excluded_urls = $this->options->get_general_setting( 'thumbsniper_exclude_urls' );

					$out.= 'var current_link = thumbsniper_rel_to_abs(link.href);';

					if( is_array( $excluded_urls ) && sizeof( $excluded_urls ) >= 1 )
					{
						foreach( $excluded_urls as $exclude )
						{
							if( $exclude == "" )
							{
								continue;
							}

							$out.= 'if((current_link.length == 0) || String(current_link).search(/^' . str_replace("/", "\\/", addslashes($exclude)) . '$/) != -1) {
									link.className=link.className + " nothumbsniper";
								}';
						}
					}else
					{
						$out.= 'if(current_link.length == 0) {
							link.className=link.className + " nothumbsniper";
						}';
					}

					//TODO: premium entfernen
					$out.= 'if(link.className.indexOf("nothumbsniper") == -1 && link.className.indexOf("thumbsniperpremium") == -1) {';

					if($thumbsniper_preview == 'external')
					{
						$out.= 'if(blogurl != linkurl) {
							link.className=link.className + " thumbsniper";
						}';
					}else
					{
						$out.= 'link.className=link.className + " thumbsniper";';
					}
			$out.= '}';
			$out.= '}else {
				link.className=link.className + " nothumbsniper"; }';
			$out.= '};';
			}

			$out.= $jQueryName . '(".thumbsniper").live("mouseover", function(event) {
				var ThumbSniperVersion = "' . $this->options->get_general_setting( "thumbsniper" ) . '";
				var ThumbSniperScaling = ' . $this->options->get_general_setting( "thumbsniper_scaling" ) . ';
				var PreviewType = "' . $this->options->get_general_setting( "thumbsniper_preview" ) . '";
				var active = true;';

			$out.= $jQueryName . '(this).qtip({
				overwrite: true,
				content: {
					text: "<div style=\"min-width: " + imageWidth + "px; min-height: " + imageHeight + "px;\">" +
						"<div class=\"thumbsniperimageholder\" style=\"width: " + imageWidth + "px; height: " + imageHeight + "px;\">" +
						"<div class=\"thumbsniperinner\">" +
						"<img class=\"thumbsniperinnerimg\" src=\"' . plugins_url('wait.gif', __FILE__) . '\"></div></div></div>",
					ajax: {
						loading: true,
						url: "' . self::GUNSLINGER_URL . '",
						type: "GET",
						dataType: "jsonp",
						data: {';
							$out.= $this->thumbsniper_json_data($jQueryName) . 
						'},
						success: function(data, status, jqXHR) {
							if(active == true) {
								if(data.url == "wait") {
									var self = this;
									setTimeout(function () {
										self.set("content.ajax.url", "' . self::GUNSLINGER_URL . '");
									}, 1000);
								}else {
									this.set("content.text", 
										"<div style=\"min-width: " + imageWidth + "px; min-height: " + imageHeight + "px;\">" +
										"<div class=\"thumbsniperimageholder\" style=\"width: " + imageWidth + "px; height: " + imageHeight + "px;\">" +
										"<div class=\"thumbsniperinner\">" +
										"<img class=\"thumbsniperinnerimg\" src=\"" + data.url + "\"></div></div></div>");
								}
							}
						}
					}
				},
				show: {
					event: event.type,
					ready: true,
					solo: true
				},
				events: {
					hide: function(event, api) {
						active = false;
					}
				},
				tip: {
					corner: true,
					border: 3,
					width: 16,
					height: 24
				},
				position: {
					my: "' . $positionMy . '",
					at: "' . $positionAt . '",
					adjust: {
						method: "flip flip",
					},
					viewport: ' . $jQueryName . '(window),
					effect: false
				},
				style: {
					classes: "ui-tooltip-' . $this->options->get_general_setting( 'thumbsniper_background' ) . ' ui-tooltip-shadow ui-tooltip-rounded",
					padding: 20
				}
			}).removeAttr("alt").removeAttr("title");
			});';

			$out.= '
				});
			});
			</script>';
		}
		$out.= "\n<!-- ThumbSniper scripts - end -->\n";
		print( $out );
	}



	function thumbsniper_json_data($jQueryName) 
	{
		$out = 'v: "' . ThumbSniper_Options::THUMBSNIPER_VERSION . '",';
		$out.= 'url: thumbsniper_rel_to_abs(' . $jQueryName . '(this).attr("href")),';
		$out.= 'size: ' . $this->options->get_general_setting( "thumbsniper_scaling" ) . ',';
		$out.= 'effect: "' . $this->options->get_general_setting( "thumbsniper_variant" ) . '"';

		return $out;
        } // function




	function thumbsniper_footer() 
	{
 		$out = "\n<!-- ThumbSniper footer - start -->\n";
		$out.= '<div style="text-align:center; font-size: xx-small; padding: 7px;">
			ThumbSniper-Plugin by <a href="http://www.mynakedgirlfriend.de" title="Thomas Schulte">Thomas Schulte</a>
			</div>';
		$out.= "\n<!-- ThumbSniper footer - end -->\n";
		print($out);
	} // function

} // class

?>
